@
*               This is a Gauss program to estimate a                   *
*  Count Data using Full Information ML and Hermite Quadrature          *
*                                                                       *
* This version: 07-11-00                                                *
************************************************************************@

new;
clear guardame,guardame2,guardame3,gbeta,ggama,grho,gsigma,gpoli;
library optmum,dutil,pgraph;
#include optmum.ext;
optset;
#include inthermi.prg;
#include hermite.prg;
#include hessian.src;

nunodes=52; /* Enter here the number of nodes for the evaluation */
           /* of the integral                                   */

{ nodes, wei } = hermite(nunodes);   /* This gives the nodes and weights */
                                     /* for the quadrature               */
@----------------------- Files names ------------------------------------@

fioutput = "fk0tzb.out";
ficoef = "fk0ctzb.fmt";
fimvc = "fk0mtzb.fmt";

dataset = "terzasc";

@------------------------- Variables names ---------------------------------@

Namesy= { tottrips };

Namesx={WORKSCHL,
        HHMEM,
        DISTOCBD,
        AREASIZE,
        FULLTIME,
        DISTONOD,
        REALINC,
        WEEKEND };  /* do not include dummy */

Namesz ={WORKSCHL,
        HHMEM,
        DISTOCBD,
        AREASIZE,
        FULLTIME,
        ADULTS,
        DISTONOD,
        REALINC };

NamesD={ OWNVEH };

NamesOtros={ BETA0, ALFA0, RHO, SIGMA, A, RHOSIG };

@----------------------- Loading Variables -------------------------------@

{y,borra1,borra2}=dgetvar(dataset,Namesy);
{x,borra1,borra2}=dgetvar(dataset,Namesx);
{d,borra1,borra2}=dgetvar(dataset,NamesD);
{z,borra1,borra2}=dgetvar(dataset,Namesz);


n=rows(x);
x=ones(n,1)~x;
z=ones(n,1)~z;
kx=cols(x)+1;  /* This one is because of the dummy */
kz=cols(z);
k=0;

@---------------------- Empieza el calculo de estadisticos ---------------@

output file = ^fioutput reset;
call dstat(dataset,0);
print "Number of nodes:";;nunodes;
print"Estimacin del modelo mediante FIML con una densidad Poisson";
print;

cls;

@---------------------- Globales en los procedures ------------------------@
cln=lnfact(y)+0.5*ln(pi); /* This is a component of the likelihood */
_opalgr=2;
_opparnm=NamesOtros[1]|namesx
    |namesd
    |namesotros[2]
    |namesz
    |NamesOtros[5:6];
__title="Direct Minimization of Minus the ML";
__output=2;
break_points={ 0,1,2,3,4,6,9,14 };

add=1;

@ *********************** Initial Conditions **************************** @

theta0={ -1.404,
         -0.339,
          0.1540,
         -0.062,
          0.040,
          0.226,
          0.022,
          0.025,
         -0.098,
          2.157,
         -0.533,
          0.326,
          0.047,
          0.676,
         -0.247,
          1.014,
          0.252,
          0.050,
          0.353,
         -0.5481,
          0.4723 };

dos cd matrices;
load theta0=fk0ctzb;
dos cd..;

@----------------------------- Minimizacion -------------------------------@

_opgtol=1e-7;
{thetaest,nf,g,retcode}=optprt(optmum(&mle,theta0));

if retcode ne 0;
print " WARNING, RETCODE IS DIFFERENT FROM ZERO ";
endif;

@-------------  We will not actually give rho but theta=rho*sigma ----------@

nf=mle(thetaest);
a=thetaest[kx+kz+2];
b=thetaest[kx+kz+1];
sigpre=sqrt(a^2+b^2); @real sigma@
rhopre=b/sigpre;  @ real rho@
thetapre=trimr(thetaest,0,2)|rhopre|sigpre;
deltajac=((a/sqrt(a^2+b^2))~(b/sqrt(a^2+b^2)))|
((-a*b*(a^2+b^2)^(-3/2))~((a^2+b^2)^(-1/2)-b^2*(a^2+b^2)^(-3/2)));

@---------------------- We compute the VCV matrix -------------------------@

add=0;
IT=gradp(&mle,thetaest);
guardame3=IT;
add=1;
IT=(1/n)*(IT'*IT);
HT=hessian(&mle,thetaest);
vcv=invpd(HT*inv(IT)*HT)/n;
vcvab=vcv[kx+kz+1:kx+kz+2,kx+kz+1:kx+kz+2];
vcvrs=deltajac*vcvab*deltajac';
std=sqrt(diag(vcv));
std=trimr(std,0,2)|sqrt(diag(vcvrs));

save ^ficoef =thetaest;
save ^fimvc =vcv;  @this save the vcm with original a and b@

@----------------------- Analisis de Autovalores -------------------------@
if minc(real(eig(ht)))<=0;
print "WARNING, Hessian with negative eigenvalues";
endif;

@----------------------- Presentacion de resultados -----------------------@

__fmtnv={"*.*lg" 15 8};
__fmtcv={"-*.*s" 12 8};

print"============================================================";
print"            Full Information Maximum Likelihood";
print"============================================================";
print"Parameter       Estimation     Std. Error         Z-Stat";
print"------------------------------------------------------------";
noms=NamesOtros[1]|namesx
    |namesd
    |namesotros[2]
    |namesz
    |NamesOtros[3:4];
call printfmt(noms~real(thetapre~std~(thetapre./std)),0~1~1~1);
print"------------------------------------------------------------";
print;

@-------------------- Estadisticos de Bondad de Ajuste --------------------@

print"Numero de parametros=";;kx+kz+2;
print"Log-Verosimilitud aproximada=";;-nf;
print"Criterio de Akaike (R)=";;2*n*nf+2*(kx+kz+2);
print"Criterio Consistente de Akaike (R2)=";;2*n*nf+(kz+kx+2)*(ln(n)+1);
print"Criterio de Schwarz BIC (R)=";;2*n*nf+(kx+kz+2)*ln(n);

@----------------- Calculamos la frecuencia esperada --------------------@

gbeta=thetapre[1:kx];
ggama=thetapre[kx+1:kx+kz];
grho=thetapre[kx+kz+1];
gsigma=thetapre[kx+kz+2];

arf=zeros(n,1);
maxy=maxc(y);
guardame=y;
pj=1;probip=zeros(n,1);

p=-1;do until p>=maxy-1; p=p+1; @this is for counts@


        y=p*ones(n,1);
        probip=probip+fajust(0)+fajust(1);

        if pj<=rows(break_points);
            if p==break_points[pj];
                arf=arf~probip;
                probip=zeros(n,1);
                pj=pj+1;
            endif;
        endif;

endo;

arf=arf[.,2:cols(arf)]~(1-sumc(arf'));
probest=meanc(arf);
y=guardame;

print;
print;
a1=rows(break_points)+1;
break_points=break_points|99*ones(1,1);
actual=counts(y,break_points)/n;

print "-----Counts------Real Number of Obs  --------Fitted Number of Obs";

fmt={ "*.*lf" 12 4,
      "*.*lf" 12 4,
      "*.*lf" 12 4 };
mask={ 1,1,1 };
mask=mask';
print "==================================================================";
results=break_points~actual~probest;
call printfm(results,mask,fmt);
print "------------------------------------------------------------------";
call printfm(sumc(results)',mask,fmt);


@------------------- Summary of goodness of fit --------------------------@

print"
--------------    Summary of goodness of fit measures    ------------------
";
print;
print "Summary measure of goodness of fit on probabilities is";
print "Taken from Cameron book, page 142-3, expression 5.33";
Print "The smallest, the best. It is always nonnegative";
sumc(abs((1/n)*(actual-probest)));

@--------------------------------------------------------------------------
    A continuacion calculamos el estadistico de ajuste (especificacion)
    del modelo tal y como aparece en el articulo de Andrews
    guardame3 contiene los gradientes en matriz NxK
---------------------------------------------------------------------------@

print"* Goodness of fit Test (Andrews, OPG version)";
print"  -------------------------------------------";
big_gamma=dummybr(y,break_points);
hache=(big_gamma-arf)~guardame3;
test=ones(n,1)'hache*invswp(hache'hache)*hache'ones(n,1);
print "Chi-Square goodness of fit= ";;test;
print "P-Value = ";;cdfchic(test,a1-1);

print"* Bondad de ajuste (no OPG)";
print"  -------------------------";

delta=0;
i=0;do until i>=n;i=i+1;

    delta=delta+guardame3[i,.]'*big_gamma[i,.];

endo;
delta=delta/n;
b=big_gamma-arf-(delta'*inv(ht)*guardame3')';
varia=(b'b)/n;
test=n*(actual-probest)'inv(varia)*(actual-probest);
print "Chi-Square goddness of fit=";;test;
print "P-Value =";;cdfchic(test,a1-1);


@ -------- Grafico de las diferencias estandarizadas --------------- @

sigma=sqrt(diag(inv(varia)));
barra=sqrt(n)*(actual-probest)./sigma;
bar(0,barra);
print"Barras diferencias";
print"==================";
barra;


dos move *.out results;
dos move *.fmt matrices;

end;

@*************************************************************************@
/************************* FIN DEL PROGRAMA *******************************/
@*************************************************************************@

/*************************************************************************
                         Funcion Objetivo
**************************************************************************/

proc mle(theta);

        local loglike,i;
        gbeta=theta[1:kx];
        ggama=theta[kx+1:kx+kz];
        grho=theta[kx+kz+1];
        gsigma=theta[kx+kz+2];
        gsigma=sqrt(gsigma^2+grho^2);
        grho=grho/gsigma;
        loglike=-Cln+ln(inthermi(&integrame,nodes,wei)+1e-7);

    if add==1;

        retp(-meanc(loglike));
    else;
        retp(-loglike);

    endif;

endp;

/***********************************************************************
Integrame: Procedimiento para calcular el valor aproximado de la Log
            likelihood. Calcula la integral sobre epsilon para una
            observacion.
************************************************************************/

proc integrame(ksi);

    local tau,a1,a2,a3,total,landa,eps,dist;
    eps=sqrt(2)*gsigma*ksi;
    tau=(z*ggama+sqrt(2)*grho*ksi)/sqrt(1-grho^2);
    dist=cdfn(tau);
    a1=d.*dist+(1-d).*(1-dist);
    landa=(x~d)*gbeta+eps;
    a2=y.*landa-exp(landa); /* due to Hermite Quadrature, we do not use */
    total=a1.*exp(a2);      /* exp(-ksi^2)                              */

    retp(total); /* Notice, the term 1/(y!*sqrt(pi)) is out of the integral */
endp;

/****************************************************************************
    fajust: Calcula la probabilidad conjunta f(y,d|w) como funcion
            de la variable binaria 'd'.
****************************************************************************/

proc fajust(de);

        local prob,i,guardame2;
        gbeta=thetapre[1:kx];
        ggama=thetapre[kx+1:kx+kz];
        grho=thetapre[kx+kz+1];
        gsigma=thetapre[kx+kz+2];
        guardame2=d;
        d=de*ones(n,1);
        prob=inthermi(&integrame,nodes,wei)./(sqrt(pi)*y!);
        d=guardame2; /* This division is because this is out of the integral */
        retp(prob);
endp;


