/* This is to evaluate the constant term of the series expansion.
We implement directly the Horner`s method (as recommended in Judd)
, so we gain accuracy and speed. We do not use GAUSS built-in
functions to evaluate polinomyals because they use loops, so they
are not so fast.

DATE 22/10/00

I have checked the results. It works well                       */


Coef_polin={ 1,
             1,1,
             1,3,1,
             1,6,7,1,
             1,10,25,15,1,
             1,15,65,90,31,1,
             1,21,140,350,301,63,1,
             1,28,266,1050,1701,966,127,1 };

/* Aadimos esta linea: cada fila del array anterior corresponde a un
grado del polinomio. La fila 'i' contiene los coeficientes del polinomio
en lambda correspondientes al momento de orden 'i' de una poisson. Los
coeficientes para el polinomio de orden 'i' estan ordenados de mayor a
menor es decir: a[1],a[2],a[3],...,a[i] son los coeficientes del
polinomio a[1]*l^i+a[2]*l^(i-1)+...+a[i-1]*l^2+a[i]*l. El coeficiente
a[i+1] (el trmino constante del polinomio es zero para cualquier momento
de la poisson.
Para obtener coeficientes de momentos adicionales de la poisson se puede
utilizar el siguiente conjunto de instrucciones en el mathematica:

m[s_]:=Exp[lambda*(Exp[s]-1)]   rem: Funcion generatriz de momentos
g[s_]:=D[m[s],{s,k}]            rem: Calcula la derivada k-esima
                                      (dar un numero a k)
Expand[g[s]/.s->0]              rem: Expande el resultado de la
                                derivada k-esima en zero
*/



/* Constante para cualquier grado del polinomio */

proc const(a,landa);

    local resultado,i,j;

    resultado=0;
    i=0;do until i>=k+1;i=i+1;

        j=0;do until j>=k+1;j=j+1;

            resultado=resultado+a[i]*a[j]*mom(i+j-2,landa);

        endo;

    endo;
retp(resultado);
endp;

/* Constante para polinomio grado 1 */

proc constan1(a,landa);
local resultado,a2sq;
a2sq=a[2]*a[2];
resultado=
          1+landa.*( 2*a[2] + a2sq +landa*a2sq);

          retp(resultado);
endp;

/* Constante para grado de polinomio 2 */

proc constan2(a,landa);

        local resultado,a2sq,a3sq,a2a3;
        a2sq=a[2]*a[2];
        a3sq=a[3]*a[3];
        a2a3=a[2]*a[3];

        resultado=  1 +
                    landa.*( a2sq+a3sq+2*a[2]+2*a2a3+2*a[3] +
                    landa.*( a2sq + 7*a3sq +6*a2a3 + 2*a[3] +
                    landa.*( 6*a3sq + 2*a[2]*a[3] +
                    landa*a3sq )));

        retp(resultado);
endp;




/* Constante para el polinomio de grado 3 */

proc constan3(a,landa);
    local resultado,a2sq,a3sq,a4sq,a2a3,a2a4,a3a4;

a2sq=a[2]*a[2];
a3sq=a[3]*a[3];
a4sq=a[4]*a[4];
a2a3=a[2]*a[3];
a2a4=a[2]*a[4];
a3a4=a[3]*a[4];
resultado=
         1+
         landa.*(2*a[3]+a2sq+2*a2a4+a3sq+a4sq+2*a[4]+2*a2a3+2*a3a4+2*a[2] +
         landa.*(2*a[3]+a2sq+14*a2a4+7*a3sq+31*a4sq+6*a[4]+6*a2a3+30*a3a4 +
         landa.*(12*a2a4+6*a3sq+90*a4sq+2*a[4]+2*a2a3+50*a3a4+
         landa.*(2*a2a4+a3sq+65*a4sq+20*a3a4+
         landa.*(15*a4sq+2*a3a4+landa*a4sq)))));

    retp(resultado);

endp;

/* Calcula la constante para un grado del polinomio de k=4 */

proc constan4(a,l); /* l works as landa */
local h1,h2,h3,h4,h5,h6,h7,h8,resultado;



          h1=2*a[2];
          h2=2*a[3]+a[2]^2;
          h3=2*a[4]+2*a[3]*a[2];
          h4=2*a[5]+2*a[2]*a[4]+a[3]^2;
          h5=2*a[2]*a[5]+2*a[3]*a[4];
          h6=2*a[3]*a[5]+a[4]^2;
          h7=2*a[4]*a[5];
          h8=a[5]*a[5];


resultado = 1+
           l.*(h1+h2+h3+h4+h5+h6+h7+h8+
           l.*(h2+3*h3+7*h4+15*h5+31*h6+63*h7+127*h8+
           l.*(h3+6*h4+25*h5+90*h6+301*h7+966*h8+
           l.*(h4+10*h5+65*h6+350*h7+1701*h8+
           l.*(h5+15*h6+140*h7+1050*h8+
           l.*(h6+21*h7+266*h8+
           l.*(h7+28*h8+l.*h8)))))));

retp(resultado);
endp;




@------------------ Calcula los momentos de una poisson -----------------@

proc mom(ka,landa);

    local aa,momento,punto_inicio,i;

    momento=0;
    if ka==0;

        momento=1;

        elseif ka==1;
        momento=landa;

        else;
        punto_inicio=sumc(seqa(1,1,ka-1));
        aa=Coef_polin[punto_inicio+1:punto_inicio+ka]|zeros(1,1);
        i=0;do until i>=ka+1;i=i+1;
            momento=momento+aa[ka-i+2]*landa^(i-1);
        endo;

    endif;

    retp(momento);

endp;

