/*
It is checked it. It works perfectly.

Returns the abscissas (x) and weights (w) for Gauss-Hermite
quadrature of order n.

This is used to compute integrals of the form
   int(exp(-x^2)f(x))
over the whole real line.
The approximation is computed as w'f(x).  The integration will be
exact
for any polynomial of order less than 2n.

Example:
  To compute the integral of the function exp(-x^2)(x^4+10) use:
  {x,w}=hermite(3);
  int=w'(x^4+10);

To convert these abscissi and weights to those appropriate for
computing moments of the standard normal distribution
multiply the abscissi by sqrt(2) and divide the weights by
sqrt(pi).

INPUT:

  N - the number of quadrature points desired

OUTPUTS:

  X - the abscissi (evaluation points)

  W - the quadrature weights



Written by Paul L. Fackler, May 1995.
There is no guarantees concerning the performance of this code.
Based on an algorithm in W.H. Press, S.A. Teukolsky, W.T.
Vetterling
and B.P. Flannery, "Numerical Recipes in FORTRAN", 2nd ed.
Cambridge
University Press, 1992.
This code may be used freely.

*/

proc (2)=hermite(n);

  local x,w,pim4,maxit,i,its,j,m,p1,p2,p3,pp,z,z1;

  maxit=100;

  pim4=1/pi^0.25;

  m=trunc((n+1)/2);

  x=zeros(n,1);

  w=zeros(n,1);

  i=0; do while i<m;i=i+1;

    @ Reasonable starting values @

    if i==1;z=sqrt(2*n+1)-1.85575*((2*n+1)^(-1/6));

      elseif i==2; z=z-1.14*(n^0.426)/z;

      elseif i==3; z=1.86*z-0.86*x[1];

      elseif i==4;z=1.91*z-0.91*x[2];

      else; z=2*z-x[i-2];

    endif;

    @ root finding iterations @

    its=0;do while its<maxit; its=its+1;

      p1=pim4;

      p2=0;

      j=0; do while j<n;j=j+1;

        p3=p2;

        p2=p1;

        p1=z.*sqrt(2/j).*p2-sqrt((j-1)/j).*p3;

      endo;

      pp=sqrt(2*n).*p2;

      z1=z;

      z=z1-p1./pp;

      if abs(z-z1)<1e-14; goto gotroot; endif;

    endo;

    "Exceeded allowable iterations in finding root"; stop;

    gotroot:

    x[n+1-i]=-z;

    x[i]=z;

    w[i]=2/(pp.*pp);

    w[n+1-i]=w[i];

  endo;

  retp(rev(x),w);

endp;

